Package: ENmix
Type: Package
Title: Quality control and analysis tools for Illumina DNA methylation
        BeadChip
Version: 1.47.3
Authors@R: 
  c(person("Zongli","Xu",email="xuz@niehs.nih.gov",role=c("cre","aut")),
    person("Liang", "Niu", role="aut",email="niulg@ucmail.uc.edu"),
    person("Jack", "Taylor",role="ctb",email="taylor@niehs.nih.gov"))
Description: Tools for quanlity control, analysis and visulization
             of Illumina DNA methylation array data. 
License: Artistic-2.0
biocViews: DNAMethylation, Preprocessing, QualityControl, TwoChannel,
        Microarray, OneChannel, MethylationArray, BatchEffect,
        Normalization, DataImport, Regression,
        PrincipalComponent,Epigenetics, MultiChannel,
        DifferentialMethylation, ImmunoOncology
Depends: parallel,doParallel,foreach,SummarizedExperiment,stats,R (>=
        3.5.0)
Imports: grDevices,graphics,matrixStats,methods,utils,irlba,
        geneplotter,impute,minfi,RPMM,illuminaio,dynamicTreeCut,IRanges,gtools,
        Biobase,ExperimentHub,AnnotationHub,genefilter,gplots,quadprog,S4Vectors
Suggests: minfiData, RUnit, BiocGenerics, BiocStyle, knitr, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://github.com/Bioconductor/ENmix
BugReports: https://github.com/Bioconductor/ENmix/issues
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/ENmix
git_branch: devel
git_last_commit: dbede0f
git_last_commit_date: 2026-04-07
Repository: Bioconductor 3.23
Date/Publication: 2026-04-07
Packaged: 2026-04-07 21:31:31 UTC; biocbuild
Author: Zongli Xu [cre, aut],
  Liang Niu [aut],
  Jack Taylor [ctb]
Maintainer: Zongli Xu <xuz@niehs.nih.gov>
