#define a few global variables
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    "DNAmFitnessModels",
    "EpiToc_CpGs",
    "GrimAgeV1_model",
    "GrimAgeV2_model",
    "GrimAge_CpG",
    "MiAgedat",
    "bAge_CpG",
    "cAge_CpG",
    "episcore_model",
    "libraryDataGet",
    "mPOA_Models",
    "methscore_dict",
    "pcc1",
    "pcc2",
    "pcc3",
    "pcc4",
    "refmeth",
    "refmeth2",
    "AdaptAge_model", 
    "CausAge_model", 
    "DamAge_model"
  ))
}

